@@ -369,6 +369,12 @@ module Gen = struct
 
   let (--.) low high = float_range ?origin:None low high
 
+  let exponential (mean : float) =
+    if Float.is_nan mean then invalid_arg "Gen.exponential";
+    let unit_gen = float_bound_inclusive 1.0 in
+    map (fun p -> -. mean *. (log p)) unit_gen
+    (* See https://en.wikipedia.org/wiki/Relationships_among_probability_distributions *)
+
   let neg_int : int t = nat >|= Int.neg
 
   (** [option gen] shrinks towards [None] then towards shrinks of [gen]. *)
@@ -791,10 +797,10 @@ module Print = struct
 
   let float = string_of_float
 
-  let bytes = Bytes.to_string
-
   let string s = Printf.sprintf "%S" s
 
+  let bytes b = string (Bytes.to_string b)
+
   let char c = Printf.sprintf "%C" c
 
   let option f = function
@@ -1169,7 +1175,6 @@ end
 (** Internal representation of functions, used for shrinking and printing (in case of error). *)
 type ('a, 'b) fun_repr_tbl = {
   fun_tbl: ('a, 'b) Poly_tbl.t; (** Input-output bindings *)
-  fun_gen: 'b Gen.t; (** How to generate output values *)
   fun_print: 'b Print.t option; (** How to print output values *)
   fun_default: 'b; (** Default value for all inputs not explicitly mapped in {!fun_tbl} *)
 }
@@ -1203,8 +1208,8 @@ module Fn = struct
 
   let make_ (r : 'a fun_repr) : 'a fun_ = Fun (r, function_of_repr r)
 
-  let mk_repr tbl gen ?print def =
-    Fun_tbl { fun_tbl=tbl; fun_gen=gen; fun_print=print; fun_default=def; }
+  let mk_repr tbl ?print def =
+    Fun_tbl { fun_tbl=tbl; fun_print=print; fun_default=def; }
 
   let map_repr f repr = Fun_map (f, repr)
 
@@ -1233,7 +1238,7 @@ module Fn = struct
   (** [gen_rep obs gen] creates a function generator. Input values are observed with [obs] and
       output values are generated with [gen]. *)
   let gen_rep (obs : 'a Observable.t) ?(print : 'b Print.t option) (gen : 'b Gen.t)  : ('a -> 'b) fun_repr Gen.t =
-    Gen.liftA2 (fun default_value poly_tbl -> mk_repr poly_tbl gen ?print default_value) gen (Poly_tbl.create ?v_print:print obs gen 8)
+    Gen.liftA2 (fun default_value poly_tbl -> mk_repr poly_tbl ?print default_value) gen (Poly_tbl.create ?v_print:print obs gen 8)
 
   let gen (obs : 'a Observable.t) ?(print : 'b Print.t option) (gen : 'b Gen.t)  : ('a -> 'b) fun_ Gen.t =
     Gen.map make_ (gen_rep obs gen ?print)
@@ -1354,9 +1359,6 @@ module TestResult = struct
     collect_tbl: (string, int) Hashtbl.t lazy_t;
     stats_tbl: ('a stat * (int, int) Hashtbl.t) list;
     mutable warnings: string list;
-    mutable instances: 'a list;
-    (** List of instances used for this test, in no particular order.
-        @since 0.9 *)
   }
 
   let get_state {state; _} = state
@@ -1391,8 +1393,6 @@ module TestResult = struct
 
   let warnings = get_warnings
 
-  let get_instances r = r.instances
-
   let is_success r = match r.state with
     | Success -> true
     | Failed _ | Error _ | Failed_other _ -> false
@@ -1567,7 +1567,7 @@ module Test = struct
     step: 'a step;
     handler : 'a handler;
     rand: RS.t;
-    mutable res: 'a TestResult.t;
+    res: 'a TestResult.t;
     mutable cur_count: int;  (** number of iterations remaining to do *)
     mutable cur_max_gen: int; (** maximum number of generations allowed *)
     mutable cur_max_fail: int; (** maximum number of counter-examples allowed *)
@@ -1763,7 +1763,6 @@ module Test = struct
   and check_state_input state input_tree =
     let Tree.Tree (input, _) = input_tree in
     state.handler state.test.name state.test (Collecting input);
-    state.res.R.instances <- input :: state.res.R.instances;
     collect state input;
     update_stats state input;
     let res =
@@ -1835,7 +1834,7 @@ module Test = struct
       res = {R.
               state=R.Success; count=0; count_gen=0;
               collect_tbl=lazy (Hashtbl.create 10);
-              instances=[]; warnings=[];
+              warnings=[];
               stats_tbl= List.map (fun stat -> stat, Hashtbl.create 10) cell.stats;
             };
     } in
